@@ -6,44 +6,91 @@
 //! unstable features are tracked in this file.
 //!
 //! If you're reading this then you're likely interested in adding a feature to
-//! Cargo, and the good news is that it shouldn't be too hard! To do this you'll
-//! want to follow these steps:
+//! Cargo, and the good news is that it shouldn't be too hard! First determine
+//! how the feature should be gated:
 //!
-//! 1. Add your feature. Do this by searching for "look here" in this file and
-//!    expanding the macro invocation that lists all features with your new
+//! * New syntax in Cargo.toml should use `cargo-features`.
+//! * New CLI options should use `-Z unstable-options`.
+//! * New functionality that may not have an interface, or the interface has
+//!   not yet been designed, or for more complex features that affect multiple
+//!   parts of Cargo should use a new `-Z` flag.
+//!
+//! See below for more details.
+//!
+//! When adding new tests for your feature, usually the tests should go into a
+//! new module of the testsuite. See
+//! <https://doc.crates.io/contrib/tests/writing.html> for more information on
+//! writing tests. Particularly, check out the "Testing Nightly Features"
+//! section for testing unstable features.
+//!
+//! After you have added your feature, be sure to update the unstable
+//! documentation at `src/doc/src/reference/unstable.md` to include a short
+//! description of how to use your new feature.
+//!
+//! And hopefully that's it!
+//!
+//! ## New Cargo.toml syntax
+//!
+//! The steps for adding new Cargo.toml syntax are:
+//!
+//! 1. Add the cargo-features unstable gate. Search below for "look here" to
+//!    find the `features!` macro and add your feature to the list.
+//!
+//! 2. Update the Cargo.toml parsing code to handle your new feature.
+//!
+//! 3. Wherever you added the new parsing code, call
+//!    `features.require(Feature::my_feature_name())?` if the new syntax is
+//!    used. This will return an error if the user hasn't listed the feature
+//!    in `cargo-features` or this is not the nightly channel.
+//!
+//! ## `-Z unstable-options`
+//!
+//! `-Z unstable-options` is intended to force the user to opt-in to new CLI
+//! flags, options, and new subcommands.
+//!
+//! The steps to add a new command-line option are:
+//!
+//! 1. Add the option to the CLI parsing code. In the help text, be sure to
+//!    include `(unstable)` to note that this is an unstable option.
+//! 2. Where the CLI option is loaded, be sure to call
+//!    [`CliUnstable::fail_if_stable_opt`]. This will return an error if `-Z
+//!    unstable options` was not passed.
+//!
+//! ## `-Z` options
+//!
+//! The steps to add a new `-Z` option are:
+//!
+//! 1. Add the option to the [`CliUnstable`] struct below. Flags can take an
+//!    optional value if you want.
+//! 2. Update the [`CliUnstable::add`] function to parse the flag.
+//! 3. Wherever the new functionality is implemented, call
+//!    [`Config::cli_unstable`][crate::util::config::Config::cli_unstable] to
+//!    get an instance of `CliUnstable` and check if the option has been
+//!    enabled on the `CliUnstable` instance. Nightly gating is already
+//!    handled, so no need to worry about that.
+//! 4. Update the `-Z help` documentation in the `main` function.
+//!
+//! ## Stabilization
+//!
+//! For the stabilization process, see
+//! <https://doc.crates.io/contrib/process/unstable.html#stabilization>.
+//!
+//! The steps for stabilizing are roughly:
+//!
+//! 1. Update the feature to be stable, based on the kind of feature:
+//!   1. `cargo-features`: Change the feature to `stable` in the `features!`
+//!      macro below.
+//!   2. `-Z unstable-options`: Find the call to `fail_if_stable_opt` and
+//!      remove it. Be sure to update the man pages if necessary.
+//!   3. `-Z` flag: Change the parsing code in [`CliUnstable::add`] to call
+//!      `stabilized_warn` or `stabilized_err`. Remove it from the `-Z help`
+//!      docs in the `main` function. Remove the `(unstable)` note in the
+//!      clap help text if necessary.
+//! 2. Remove `masquerade_as_nightly_cargo` from any tests, and remove
+//!    `cargo-features` from `Cargo.toml` test files if any.
+//! 3. Remove the docs from unstable.md and update the redirect at the bottom
+//!    of that page. Update the rest of the documentation to add the new
 //!    feature.
-//!
-//! 2. Find the appropriate place to place the feature gate in Cargo itself. If
-//!    you're extending the manifest format you'll likely just want to modify
-//!    the `Manifest::feature_gate` function, but otherwise you may wish to
-//!    place the feature gate elsewhere in Cargo.
-//!
-//! 3. To actually perform the feature gate, you'll want to have code that looks
-//!    like:
-//!
-//! ```rust,compile_fail
-//! use core::{Feature, Features};
-//!
-//! let feature = Feature::launch_into_space();
-//! package.manifest().unstable_features().require(feature).chain_err(|| {
-//!     "launching Cargo into space right now is unstable and may result in \
-//!      unintended damage to your codebase, use with caution"
-//! })?;
-//! ```
-//!
-//! Notably you'll notice the `require` function called with your `Feature`, and
-//! then you use `chain_err` to tack on more context for why the feature was
-//! required when the feature isn't activated.
-//!
-//! 4. Update the unstable documentation at
-//!    `src/doc/src/reference/unstable.md` to include a short description of
-//!    how to use your new feature. When the feature is stabilized, be sure
-//!    that the Cargo Guide or Reference is updated to fully document the
-//!    feature and remove the entry from the Unstable section.
-//!
-//! And hopefully that's it! Bear with us though that this is, at the time of
-//! this writing, a very new feature in Cargo. If the process differs from this
-//! we'll be sure to update this documentation!
 
 use std::cell::Cell;
 use std::env;
@@ -54,6 +101,7 @@ use anyhow::{bail, Error};
 use serde::{Deserialize, Serialize};
 
 use crate::util::errors::CargoResult;
+use crate::util::indented_lines;
 
 pub const SEE_CHANNELS: &str =
     "See https://doc.rust-lang.org/book/appendix-07-nightly-rust.html for more information \
@@ -66,6 +114,19 @@ pub enum Edition {
     Edition2015,
     /// The 2018 edition
     Edition2018,
+    /// The 2021 edition
+    Edition2021,
+}
+
+impl Edition {
+    pub(crate) fn first_version(&self) -> Option<semver::Version> {
+        use Edition::*;
+        match self {
+            Edition2015 => None,
+            Edition2018 => Some(semver::Version::new(1, 31, 0)),
+            Edition2021 => Some(semver::Version::new(1, 62, 0)),
+        }
+    }
 }
 
 impl fmt::Display for Edition {
@@ -73,6 +134,7 @@ impl fmt::Display for Edition {
         match *self {
             Edition::Edition2015 => f.write_str("2015"),
             Edition::Edition2018 => f.write_str("2018"),
+            Edition::Edition2021 => f.write_str("2021"),
         }
     }
 }
@@ -82,14 +144,15 @@ impl FromStr for Edition {
         match s {
             "2015" => Ok(Edition::Edition2015),
             "2018" => Ok(Edition::Edition2018),
-            s if s.parse().map_or(false, |y: u16| y > 2020 && y < 2050) => bail!(
+            "2021" => Ok(Edition::Edition2021),
+            s if s.parse().map_or(false, |y: u16| y > 2021 && y < 2050) => bail!(
                 "this version of Cargo is older than the `{}` edition, \
-                 and only supports `2015` and `2018` editions.",
+                 and only supports `2015`, `2018`, and `2021` editions.",
                 s
             ),
             s => bail!(
-                "supported edition values are `2015` or `2018`, but `{}` \
-                 is unknown",
+                "supported edition values are `2015`, `2018`, or `2021`, \
+                 but `{}` is unknown",
                 s
             ),
         }
@@ -100,13 +163,12 @@ impl FromStr for Edition {
 enum Status {
     Stable,
     Unstable,
+    Removed,
 }
 
 macro_rules! features {
     (
-        pub struct Features {
-            $([$stab:ident] $feature:ident: bool,)*
-        }
+        $(($stab:ident, $feature:ident, $version:expr, $docs:expr),)*
     ) => (
         #[derive(Default, Clone, Debug)]
         pub struct Features {
@@ -122,6 +184,9 @@ macro_rules! features {
                     }
                     static FEAT: Feature = Feature {
                         name: stringify!($feature),
+                        stability: stab!($stab),
+                        version: $version,
+                        docs: $docs,
                         get,
                     };
                     &FEAT
@@ -134,14 +199,14 @@ macro_rules! features {
         }
 
         impl Features {
-            fn status(&mut self, feature: &str) -> Option<(&mut bool, Status)> {
+            fn status(&mut self, feature: &str) -> Option<(&mut bool, &'static Feature)> {
                 if feature.contains("_") {
                     return None
                 }
                 let feature = feature.replace("-", "_");
                 $(
                     if feature == stringify!($feature) {
-                        return Some((&mut self.$feature, stab!($stab)))
+                        return Some((&mut self.$feature, Feature::$feature()))
                     }
                 )*
                 None
@@ -157,6 +222,9 @@ macro_rules! stab {
     (unstable) => {
         Status::Unstable
     };
+    (removed) => {
+        Status::Removed
+    };
 }
 
 // A listing of all features in Cargo.
@@ -171,54 +239,64 @@ macro_rules! stab {
 // character is translated to `-` when specified in the `cargo-features`
 // manifest entry in `Cargo.toml`.
 features! {
-    pub struct Features {
+    // A dummy feature that doesn't actually gate anything, but it's used in
+    // testing to ensure that we can enable stable features.
+    (stable, test_dummy_stable, "1.0", ""),
 
-        // A dummy feature that doesn't actually gate anything, but it's used in
-        // testing to ensure that we can enable stable features.
-        [stable] test_dummy_stable: bool,
+    // A dummy feature that gates the usage of the `im-a-teapot` manifest
+    // entry. This is basically just intended for tests.
+    (unstable, test_dummy_unstable, "", "reference/unstable.html"),
 
-        // A dummy feature that gates the usage of the `im-a-teapot` manifest
-        // entry. This is basically just intended for tests.
-        [unstable] test_dummy_unstable: bool,
+    // Downloading packages from alternative registry indexes.
+    (stable, alternative_registries, "1.34", "reference/registries.html"),
 
-        // Downloading packages from alternative registry indexes.
-        [stable] alternative_registries: bool,
+    // Using editions
+    (stable, edition, "1.31", "reference/manifest.html#the-edition-field"),
 
-        // Using editions
-        [stable] edition: bool,
+    // Renaming a package in the manifest via the `package` key
+    (stable, rename_dependency, "1.31", "reference/specifying-dependencies.html#renaming-dependencies-in-cargotoml"),
 
-        // Renaming a package in the manifest via the `package` key
-        [stable] rename_dependency: bool,
+    // Whether a lock file is published with this crate
+    (removed, publish_lockfile, "", PUBLISH_LOCKFILE_REMOVED),
 
-        // Whether a lock file is published with this crate
-        // This is deprecated, and will likely be removed in a future version.
-        [unstable] publish_lockfile: bool,
+    // Overriding profiles for dependencies.
+    (stable, profile_overrides, "1.41", "reference/profiles.html#overrides"),
 
-        // Overriding profiles for dependencies.
-        [stable] profile_overrides: bool,
+    // "default-run" manifest option,
+    (stable, default_run, "1.37", "reference/manifest.html#the-default-run-field"),
 
-        // "default-run" manifest option,
-        [stable] default_run: bool,
+    // Declarative build scripts.
+    (unstable, metabuild, "", "reference/unstable.html#metabuild"),
 
-        // Declarative build scripts.
-        [unstable] metabuild: bool,
+    // Specifying the 'public' attribute on dependencies
+    (unstable, public_dependency, "", "reference/unstable.html#public-dependency"),
 
-        // Specifying the 'public' attribute on dependencies
-        [unstable] public_dependency: bool,
+    // Allow to specify profiles other than 'dev', 'release', 'test', etc.
+    (unstable, named_profiles, "", "reference/unstable.html#custom-named-profiles"),
 
-        // Allow to specify profiles other than 'dev', 'release', 'test', etc.
-        [unstable] named_profiles: bool,
+    // Opt-in new-resolver behavior.
+    (stable, resolver, "1.51", "reference/resolver.html#resolver-versions"),
 
-        // Opt-in new-resolver behavior.
-        [unstable] resolver: bool,
+    // Allow to specify whether binaries should be stripped.
+    (unstable, strip, "", "reference/unstable.html#profile-strip-option"),
 
-        // Allow to specify whether binaries should be stripped.
-        [unstable] strip: bool,
-    }
+    // Specifying a minimal 'rust-version' attribute for crates
+    (unstable, rust_version, "", "reference/unstable.html#rust-version"),
 }
 
+const PUBLISH_LOCKFILE_REMOVED: &str = "The publish-lockfile key in Cargo.toml \
+    has been removed. The Cargo.lock file is always included when a package is \
+    published if the package contains a binary target. `cargo install` requires \
+    the `--locked` flag to use the Cargo.lock file.\n\
+    See https://doc.rust-lang.org/cargo/commands/cargo-package.html and \
+    https://doc.rust-lang.org/cargo/commands/cargo-install.html for more \
+    information.";
+
 pub struct Feature {
     name: &'static str,
+    stability: Status,
+    version: &'static str,
+    docs: &'static str,
     get: fn(&Features) -> bool,
 }
 
@@ -232,35 +310,61 @@ impl Features {
         Ok(ret)
     }
 
-    fn add(&mut self, feature: &str, warnings: &mut Vec<String>) -> CargoResult<()> {
-        let (slot, status) = match self.status(feature) {
+    fn add(&mut self, feature_name: &str, warnings: &mut Vec<String>) -> CargoResult<()> {
+        let (slot, feature) = match self.status(feature_name) {
             Some(p) => p,
-            None => bail!("unknown cargo feature `{}`", feature),
+            None => bail!("unknown cargo feature `{}`", feature_name),
         };
 
         if *slot {
-            bail!("the cargo feature `{}` has already been activated", feature);
+            bail!(
+                "the cargo feature `{}` has already been activated",
+                feature_name
+            );
         }
 
-        match status {
+        let see_docs = || {
+            let url_channel = match channel().as_str() {
+                "dev" | "nightly" => "nightly/",
+                "beta" => "beta/",
+                _ => "",
+            };
+            format!(
+                "See https://doc.rust-lang.org/{}cargo/{} for more information \
+                about using this feature.",
+                url_channel, feature.docs
+            )
+        };
+
+        match feature.stability {
             Status::Stable => {
                 let warning = format!(
-                    "the cargo feature `{}` is now stable \
-                     and is no longer necessary to be listed \
-                     in the manifest",
-                    feature
+                    "the cargo feature `{}` has been stabilized in the {} \
+                     release and is no longer necessary to be listed in the \
+                     manifest\n  {}",
+                    feature_name,
+                    feature.version,
+                    see_docs()
                 );
                 warnings.push(warning);
             }
             Status::Unstable if !nightly_features_allowed() => bail!(
                 "the cargo feature `{}` requires a nightly version of \
                  Cargo, but this is the `{}` channel\n\
-                 {}",
-                feature,
+                 {}\n{}",
+                feature_name,
                 channel(),
-                SEE_CHANNELS
+                SEE_CHANNELS,
+                see_docs()
             ),
             Status::Unstable => {}
+            Status::Removed => bail!(
+                "the cargo feature `{}` has been removed\n\
+                Remove the feature from Cargo.toml to remove this error.\n\
+                {}",
+                feature_name,
+                feature.docs
+            ),
         }
 
         *slot = true;
@@ -310,26 +414,6 @@ impl Features {
 /// Cargo, like `rustc`, accepts a suite of `-Z` flags which are intended for
 /// gating unstable functionality to Cargo. These flags are only available on
 /// the nightly channel of Cargo.
-///
-/// This struct doesn't have quite the same convenience macro that the features
-/// have above, but the procedure should still be relatively stable for adding a
-/// new unstable flag:
-///
-/// 1. First, add a field to this `CliUnstable` structure. All flags are allowed
-///    to have a value as the `-Z` flags are either of the form `-Z foo` or
-///    `-Z foo=bar`, and it's up to you how to parse `bar`.
-///
-/// 2. Add an arm to the match statement in `CliUnstable::add` below to match on
-///    your new flag. The key (`k`) is what you're matching on and the value is
-///    in `v`.
-///
-/// 3. (optional) Add a new parsing function to parse your datatype. As of now
-///    there's an example for `bool`, but more can be added!
-///
-/// 4. In Cargo use `config.cli_unstable()` to get a reference to this structure
-///    and then test for your flag or your value and act accordingly.
-///
-/// If you have any trouble with this, please let us know!
 #[derive(Default, Debug, Deserialize)]
 #[serde(default, rename_all = "kebab-case")]
 pub struct CliUnstable {
@@ -338,7 +422,6 @@ pub struct CliUnstable {
     pub no_index_update: bool,
     pub avoid_dev_deps: bool,
     pub minimal_versions: bool,
-    pub package_features: bool,
     pub advanced_env: bool,
     pub config_include: bool,
     pub dual_proc_macros: bool,
@@ -363,6 +446,40 @@ pub struct CliUnstable {
     pub credential_process: bool,
 }
 
+const STABILIZED_COMPILE_PROGRESS: &str = "The progress bar is now always \
+    enabled when used on an interactive console.\n\
+    See https://doc.rust-lang.org/cargo/reference/config.html#termprogresswhen \
+    for information on controlling the progress bar.";
+
+const STABILIZED_OFFLINE: &str = "Offline mode is now available via the \
+    --offline CLI option";
+
+const STABILIZED_CACHE_MESSAGES: &str = "Message caching is now always enabled.";
+
+const STABILIZED_INSTALL_UPGRADE: &str = "Packages are now always upgraded if \
+    they appear out of date.\n\
+    See https://doc.rust-lang.org/cargo/commands/cargo-install.html for more \
+    information on how upgrading works.";
+
+const STABILIZED_CONFIG_PROFILE: &str = "See \
+    https://doc.rust-lang.org/cargo/reference/config.html#profile for more \
+    information about specifying profiles in config.";
+
+const STABILIZED_CRATE_VERSIONS: &str = "The crate version is now \
+    automatically added to the documentation.";
+
+const STABILIZED_PACKAGE_FEATURES: &str = "Enhanced feature flag behavior is now \
+    available in virtual workspaces, and `member/feature-name` syntax is also \
+    always available. Other extensions require setting `resolver = \"2\"` in \
+    Cargo.toml.\n\
+    See https://doc.rust-lang.org/nightly/cargo/reference/features.html#resolver-version-2-command-line-flags \
+    for more information.";
+
+const STABILIZED_FEATURES: &str = "The new feature resolver is now available \
+    by specifying `resolver = \"2\"` in Cargo.toml.\n\
+    See https://doc.rust-lang.org/nightly/cargo/reference/features.html#feature-resolver-version-2 \
+    for more information.";
+
 fn deserialize_build_std<'de, D>(deserializer: D) -> Result<Option<Vec<String>>, D::Error>
 where
     D: serde::Deserializer<'de>,
@@ -378,7 +495,7 @@ where
 }
 
 impl CliUnstable {
-    pub fn parse(&mut self, flags: &[String]) -> CargoResult<()> {
+    pub fn parse(&mut self, flags: &[String]) -> CargoResult<Vec<String>> {
         if !flags.is_empty() && !nightly_features_allowed() {
             bail!(
                 "the `-Z` flag is only accepted on the nightly channel of Cargo, \
@@ -388,13 +505,14 @@ impl CliUnstable {
                 SEE_CHANNELS
             );
         }
+        let mut warnings = Vec::new();
         for flag in flags {
-            self.add(flag)?;
+            self.add(flag, &mut warnings)?;
         }
-        Ok(())
+        Ok(warnings)
     }
 
-    fn add(&mut self, flag: &str) -> CargoResult<()> {
+    fn add(&mut self, flag: &str, warnings: &mut Vec<String>) -> CargoResult<()> {
         let mut parts = flag.splitn(2, '=');
         let k = parts.next().unwrap();
         let v = parts.next();
@@ -439,13 +557,32 @@ impl CliUnstable {
             })
         }
 
+        let mut stabilized_warn = |key: &str, version: &str, message: &str| {
+            warnings.push(format!(
+                "flag `-Z {}` has been stabilized in the {} release, \
+                 and is no longer necessary\n{}",
+                key,
+                version,
+                indented_lines(message)
+            ));
+        };
+
+        // Use this if the behavior now requires another mechanism to enable.
+        let stabilized_err = |key: &str, version: &str, message: &str| {
+            Err(anyhow::format_err!(
+                "flag `-Z {}` has been stabilized in the {} release\n{}",
+                key,
+                version,
+                indented_lines(message)
+            ))
+        };
+
         match k {
             "print-im-a-teapot" => self.print_im_a_teapot = parse_bool(k, v)?,
             "unstable-options" => self.unstable_options = parse_empty(k, v)?,
             "no-index-update" => self.no_index_update = parse_empty(k, v)?,
             "avoid-dev-deps" => self.avoid_dev_deps = parse_empty(k, v)?,
             "minimal-versions" => self.minimal_versions = parse_empty(k, v)?,
-            "package-features" => self.package_features = parse_empty(k, v)?,
             "advanced-env" => self.advanced_env = parse_empty(k, v)?,
             "config-include" => self.config_include = parse_empty(k, v)?,
             "dual-proc-macros" => self.dual_proc_macros = parse_empty(k, v)?,
@@ -461,7 +598,27 @@ impl CliUnstable {
             "doctest-xcompile" => self.doctest_xcompile = parse_empty(k, v)?,
             "panic-abort-tests" => self.panic_abort_tests = parse_empty(k, v)?,
             "jobserver-per-rustc" => self.jobserver_per_rustc = parse_empty(k, v)?,
-            "features" => self.features = Some(parse_features(v)),
+            "features" => {
+                // For now this is still allowed (there are still some
+                // unstable options like "compare"). This should be removed at
+                // some point, and migrate to a new -Z flag for any future
+                // things.
+                let feats = parse_features(v);
+                let stab: Vec<_> = feats
+                    .iter()
+                    .filter(|feat| {
+                        matches!(
+                            feat.as_str(),
+                            "build_dep" | "host_dep" | "dev_dep" | "itarget" | "all"
+                        )
+                    })
+                    .collect();
+                if !stab.is_empty() || feats.is_empty() {
+                    // Make this stabilized_err once -Zfeature support is removed.
+                    stabilized_warn(k, "1.51", STABILIZED_FEATURES);
+                }
+                self.features = Some(feats);
+            }
             "separate-nightlies" => self.separate_nightlies = parse_empty(k, v)?,
             "multitarget" => self.multitarget = parse_empty(k, v)?,
             "rustdoc-map" => self.rustdoc_map = parse_empty(k, v)?,
@@ -470,6 +627,13 @@ impl CliUnstable {
             "weak-dep-features" => self.weak_dep_features = parse_empty(k, v)?,
             "extra-link-arg" => self.extra_link_arg = parse_empty(k, v)?,
             "credential-process" => self.credential_process = parse_empty(k, v)?,
+            "compile-progress" => stabilized_warn(k, "1.30", STABILIZED_COMPILE_PROGRESS),
+            "offline" => stabilized_err(k, "1.36", STABILIZED_OFFLINE)?,
+            "cache-messages" => stabilized_warn(k, "1.40", STABILIZED_CACHE_MESSAGES),
+            "install-upgrade" => stabilized_warn(k, "1.41", STABILIZED_INSTALL_UPGRADE),
+            "config-profile" => stabilized_warn(k, "1.43", STABILIZED_CONFIG_PROFILE),
+            "crate-versions" => stabilized_warn(k, "1.47", STABILIZED_CRATE_VERSIONS),
+            "package-features" => stabilized_warn(k, "1.51", STABILIZED_PACKAGE_FEATURES),
             _ => bail!("unknown `-Z` flag specified: {}", k),
         }
 
