@@ -4,6 +4,7 @@ use crate::core::{PackageIdSpec, TargetKind, Workspace};
 use crate::ops;
 use crate::util::errors::{CargoResult, CargoResultExt};
 use crate::util::interning::InternedString;
+use crate::util::lev_distance;
 use crate::util::paths;
 use crate::util::Config;
 use std::fs;
@@ -34,12 +35,7 @@ pub fn clean(ws: &Workspace<'_>, opts: &CleanOptions<'_>) -> CargoResult<()> {
         return rm_rf(&target_dir.into_path_unlocked(), config);
     }
 
-    let profiles = Profiles::new(
-        ws.profiles(),
-        config,
-        opts.requested_profile,
-        ws.unstable_features(),
-    )?;
+    let profiles = Profiles::new(ws, opts.requested_profile)?;
 
     if opts.profile_specified {
         // After parsing profiles we know the dir-name of the profile, if a profile
@@ -119,7 +115,17 @@ pub fn clean(ws: &Workspace<'_>, opts: &CleanOptions<'_>) -> CargoResult<()> {
         }
         let matches: Vec<_> = resolve.iter().filter(|id| spec.matches(*id)).collect();
         if matches.is_empty() {
-            anyhow::bail!("package ID specification `{}` matched no packages", spec);
+            let mut suggestion = String::new();
+            suggestion.push_str(&lev_distance::closest_msg(
+                &spec.name(),
+                resolve.iter(),
+                |id| id.name().as_str(),
+            ));
+            anyhow::bail!(
+                "package ID specification `{}` did not match any packages{}",
+                spec,
+                suggestion
+            );
         }
         pkg_ids.extend(matches);
     }
@@ -170,9 +176,14 @@ pub fn clean(ws: &Workspace<'_>, opts: &CleanOptions<'_>) -> CargoResult<()> {
                         // Remove dep-info file generated by rustc. It is not tracked in
                         // file_types. It does not have a prefix.
                         let hashed_dep_info = dir.join(format!("{}-*.d", crate_name));
-                        let unhashed_dep_info = dir.join(format!("{}.d", crate_name));
                         rm_rf_glob(&hashed_dep_info, config)?;
+                        let unhashed_dep_info = dir.join(format!("{}.d", crate_name));
                         rm_rf(&unhashed_dep_info, config)?;
+                        // Remove split-debuginfo files generated by rustc.
+                        let split_debuginfo_obj = dir.join(format!("{}.*.o", crate_name));
+                        rm_rf_glob(&split_debuginfo_obj, config)?;
+                        let split_debuginfo_dwo = dir.join(format!("{}.*.dwo", crate_name));
+                        rm_rf_glob(&split_debuginfo_dwo, config)?;
 
                         // Remove the uplifted copy.
                         if let Some(uplift_dir) = uplift_dir {
