@@ -20,6 +20,7 @@ mod build;
 mod build_plan;
 mod build_script;
 mod build_script_env;
+mod build_script_extra_link_arg;
 mod cache_messages;
 mod cargo_alias_config;
 mod cargo_command;
@@ -34,6 +35,7 @@ mod config;
 mod config_cli;
 mod config_include;
 mod corrupt_git;
+mod credential_process;
 mod cross_compile;
 mod cross_publish;
 mod custom_target;
@@ -45,6 +47,7 @@ mod edition;
 mod error;
 mod features;
 mod features2;
+mod features_namespaced;
 mod fetch;
 mod fix;
 mod freshness;
@@ -52,16 +55,20 @@ mod generate_lockfile;
 mod git;
 mod git_auth;
 mod git_gc;
+mod glob_targets;
+mod help;
 mod init;
 mod install;
 mod install_upgrade;
 mod jobserver;
-mod list_targets;
+mod list_availables;
 mod local_registry;
 mod locate_project;
 mod lockfile_compat;
 mod login;
+mod logout;
 mod lto;
+mod member_discovery;
 mod member_errors;
 mod message_format;
 mod metabuild;
@@ -86,6 +93,7 @@ mod profile_custom;
 mod profile_overrides;
 mod profile_targets;
 mod profiles;
+mod progress;
 mod pub_priv;
 mod publish;
 mod publish_lockfile;
@@ -95,6 +103,7 @@ mod rename_deps;
 mod replace;
 mod required_features;
 mod run;
+mod rust_version;
 mod rustc;
 mod rustc_info_cache;
 mod rustdoc;
@@ -115,6 +124,7 @@ mod vendor;
 mod verify_project;
 mod version;
 mod warn_on_failure;
+mod weak_dep_features;
 mod workspaces;
 mod yank;
 
