@@ -27,7 +27,7 @@ fn named_profile_gated() {
 Caused by:
   feature `named-profiles` is required
 
-consider adding `cargo-features = [\"named-profiles\"]` to the manifest
+  consider adding `cargo-features = [\"named-profiles\"]` to the manifest
 ",
         )
         .with_status(101)
@@ -42,21 +42,21 @@ fn profile_config_validate_warnings() {
         .file(
             ".cargo/config",
             r#"
-            [profile.test]
-            opt-level = 3
+                [profile.test]
+                opt-level = 3
 
-            [profile.asdf]
-            opt-level = 3
+                [profile.asdf]
+                opt-level = 3
 
-            [profile.dev]
-            bad-key = true
+                [profile.dev]
+                bad-key = true
 
-            [profile.dev.build-override]
-            bad-key-bo = true
+                [profile.dev.build-override]
+                bad-key-bo = true
 
-            [profile.dev.package.bar]
-            bad-key-bar = true
-        "#,
+                [profile.dev.package.bar]
+                bad-key-bar = true
+            "#,
         )
         .build();
 
@@ -82,9 +82,9 @@ fn profile_config_error_paths() {
         .file(
             ".cargo/config",
             r#"
-            [profile.dev]
-            opt-level = 3
-        "#,
+                [profile.dev]
+                opt-level = 3
+            "#,
         )
         .file(
             paths::home().join(".cargo/config"),
@@ -116,9 +116,9 @@ fn profile_config_validate_errors() {
         .file(
             ".cargo/config",
             r#"
-            [profile.dev.package.foo]
-            panic = "abort"
-        "#,
+                [profile.dev.package.foo]
+                panic = "abort"
+            "#,
         )
         .build();
 
@@ -143,9 +143,9 @@ fn profile_config_syntax_errors() {
         .file(
             ".cargo/config",
             r#"
-            [profile.dev]
-            codegen-units = "foo"
-        "#,
+                [profile.dev]
+                codegen-units = "foo"
+            "#,
         )
         .build();
 
@@ -179,12 +179,12 @@ fn profile_config_override_spec_multiple() {
         .file(
             ".cargo/config",
             r#"
-            [profile.dev.package.bar]
-            opt-level = 3
+                [profile.dev.package.bar]
+                opt-level = 3
 
-            [profile.dev.package."bar:0.5.0"]
-            opt-level = 3
-        "#,
+                [profile.dev.package."bar:0.5.0"]
+                opt-level = 3
+            "#,
         )
         .file("src/lib.rs", "")
         .file("bar/Cargo.toml", &basic_lib_manifest("bar"))
@@ -211,17 +211,17 @@ fn profile_config_all_options() {
         .file(
             ".cargo/config",
             r#"
-        [profile.release]
-        opt-level = 1
-        debug = true
-        debug-assertions = true
-        overflow-checks = false
-        rpath = true
-        lto = true
-        codegen-units = 2
-        panic = "abort"
-        incremental = true
-        "#,
+            [profile.release]
+            opt-level = 1
+            debug = true
+            debug-assertions = true
+            overflow-checks = false
+            rpath = true
+            lto = true
+            codegen-units = 2
+            panic = "abort"
+            incremental = true
+            "#,
         )
         .build();
 
@@ -233,7 +233,7 @@ fn profile_config_all_options() {
 [RUNNING] `rustc --crate-name foo [..] \
             -C opt-level=1 \
             -C panic=abort \
-            -C lto \
+            -C lto[..]\
             -C codegen-units=2 \
             -C debuginfo=2 \
             -C debug-assertions=on \
@@ -253,19 +253,19 @@ fn profile_config_override_precedence() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
+                [package]
+                name = "foo"
+                version = "0.0.1"
 
-            [dependencies]
-            bar = {path = "bar"}
+                [dependencies]
+                bar = {path = "bar"}
 
-            [profile.dev]
-            codegen-units = 2
+                [profile.dev]
+                codegen-units = 2
 
-            [profile.dev.package.bar]
-            opt-level = 3
-        "#,
+                [profile.dev.package.bar]
+                opt-level = 3
+            "#,
         )
         .file("src/lib.rs", "")
         .file("bar/Cargo.toml", &basic_lib_manifest("bar"))
@@ -273,9 +273,9 @@ fn profile_config_override_precedence() {
         .file(
             ".cargo/config",
             r#"
-            [profile.dev.package.bar]
-            opt-level = 2
-        "#,
+                [profile.dev.package.bar]
+                opt-level = 2
+            "#,
         )
         .build();
 
@@ -283,7 +283,7 @@ fn profile_config_override_precedence() {
         .with_stderr(
             "\
 [COMPILING] bar [..]
-[RUNNING] `rustc --crate-name bar [..] -C opt-level=2 -C codegen-units=2 [..]
+[RUNNING] `rustc --crate-name bar [..] -C opt-level=2[..]-C codegen-units=2 [..]
 [COMPILING] foo [..]
 [RUNNING] `rustc --crate-name foo [..]-C codegen-units=2 [..]
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]",
@@ -299,9 +299,9 @@ fn profile_config_no_warn_unknown_override() {
         .file(
             ".cargo/config",
             r#"
-            [profile.dev.package.bar]
-            codegen-units = 4
-        "#,
+                [profile.dev.package.bar]
+                codegen-units = 4
+            "#,
         )
         .build();
 
@@ -318,9 +318,9 @@ fn profile_config_mixed_types() {
         .file(
             ".cargo/config",
             r#"
-            [profile.dev]
-            opt-level = 3
-        "#,
+                [profile.dev]
+                opt-level = 3
+            "#,
         )
         .file(
             paths::home().join(".cargo/config"),
@@ -344,68 +344,68 @@ fn named_config_profile() {
     use super::config::ConfigBuilder;
     use cargo::core::compiler::CompileMode;
     use cargo::core::enable_nightly_features;
-    use cargo::core::features::Features;
     use cargo::core::profiles::{Profiles, UnitFor};
-    use cargo::core::{InternedString, PackageId};
-    use cargo::util::toml::TomlProfiles;
+    use cargo::core::{PackageId, Workspace};
+    use cargo::util::interning::InternedString;
     use std::fs;
     enable_nightly_features();
     paths::root().join(".cargo").mkdir_p();
     fs::write(
         paths::root().join(".cargo/config"),
         r#"
-        [profile.foo]
-        inherits = "middle"
-        codegen-units = 2
-        [profile.foo.build-override]
-        codegen-units = 6
-        [profile.foo.package.dep]
-        codegen-units = 7
+            [profile.foo]
+            inherits = "middle"
+            codegen-units = 2
+            [profile.foo.build-override]
+            codegen-units = 6
+            [profile.foo.package.dep]
+            codegen-units = 7
 
-        [profile.middle]
-        inherits = "bar"
-        codegen-units = 3
+            [profile.middle]
+            inherits = "bar"
+            codegen-units = 3
 
-        [profile.bar]
-        inherits = "dev"
-        codegen-units = 4
-        debug = 1
+            [profile.bar]
+            inherits = "dev"
+            codegen-units = 4
+            debug = 1
+        "#,
+    )
+    .unwrap();
+    fs::write(
+        paths::root().join("Cargo.toml"),
+        r#"
+            cargo-features = ['named-profiles']
+
+            [workspace]
+
+            [profile.middle]
+            inherits = "bar"
+            codegen-units = 1
+            opt-level = 1
+            [profile.middle.package.dep]
+            overflow-checks = false
+
+            [profile.foo.build-override]
+            codegen-units = 5
+            debug-assertions = false
+            [profile.foo.package.dep]
+            codegen-units = 8
         "#,
     )
     .unwrap();
     let config = ConfigBuilder::new().build();
-    let mut warnings = Vec::new();
-    let features = Features::new(&["named-profiles".to_string()], &mut warnings).unwrap();
-    assert_eq!(warnings.len(), 0);
     let profile_name = InternedString::new("foo");
-    let toml = r#"
-        [profile.middle]
-        inherits = "bar"
-        codegen-units = 1
-        opt-level = 1
-        [profile.middle.package.dep]
-        overflow-checks = false
-
-        [profile.foo.build-override]
-        codegen-units = 5
-        debug-assertions = false
-        [profile.foo.package.dep]
-        codegen-units = 8
-    "#;
-    #[derive(serde::Deserialize)]
-    struct TomlManifest {
-        profile: Option<TomlProfiles>,
-    }
-    let manifest: TomlManifest = toml::from_str(toml).unwrap();
-    let profiles =
-        Profiles::new(manifest.profile.as_ref(), &config, profile_name, &features).unwrap();
+    let ws = Workspace::new(&paths::root().join("Cargo.toml"), &config).unwrap();
+    let profiles = Profiles::new(&ws, profile_name).unwrap();
 
     let crates_io = cargo::core::source::SourceId::crates_io(&config).unwrap();
     let a_pkg = PackageId::new("a", "0.1.0", crates_io).unwrap();
     let dep_pkg = PackageId::new("dep", "0.1.0", crates_io).unwrap();
 
     // normal package
-    let p = profiles.get_profile(a_pkg, true, UnitFor::new_normal(), CompileMode::Build);
+    let mode = CompileMode::Build;
+    let p = profiles.get_profile(a_pkg, true, true, UnitFor::new_normal(), mode);
     assert_eq!(p.name, "foo");
     assert_eq!(p.codegen_units, Some(2)); // "foo" from config
     assert_eq!(p.opt_level, "1"); // "middle" from manifest
@@ -414,16 +414,16 @@ fn named_config_profile() {
     assert_eq!(p.overflow_checks, true); // "dev" built-in (ignore package override)
 
     // build-override
-    let bo = profiles.get_profile(a_pkg, true, UnitFor::new_build(false), CompileMode::Build);
+    let bo = profiles.get_profile(a_pkg, true, true, UnitFor::new_host(false), mode);
     assert_eq!(bo.name, "foo");
     assert_eq!(bo.codegen_units, Some(6)); // "foo" build override from config
-    assert_eq!(bo.opt_level, "1"); // SAME as normal
+    assert_eq!(bo.opt_level, "0"); // default to zero
     assert_eq!(bo.debuginfo, Some(1)); // SAME as normal
     assert_eq!(bo.debug_assertions, false); // "foo" build override from manifest
     assert_eq!(bo.overflow_checks, true); // SAME as normal
 
     // package overrides
-    let po = profiles.get_profile(dep_pkg, false, UnitFor::new_normal(), CompileMode::Build);
+    let po = profiles.get_profile(dep_pkg, false, true, UnitFor::new_normal(), mode);
     assert_eq!(po.name, "foo");
     assert_eq!(po.codegen_units, Some(7)); // "foo" package override from config
     assert_eq!(po.opt_level, "1"); // SAME as normal
