@@ -1,9 +1,12 @@
+//! Tests for public/private dependencies.
+
 use cargo_test_support::registry::Package;
 use cargo_test_support::{is_nightly, project};
 
 #[cargo_test]
 fn exported_priv_warning() {
     if !is_nightly() {
+        // exported_private_dependencies lint is unstable
         return;
     }
     Package::new("priv_dep", "0.1.0")
@@ -14,15 +17,15 @@ fn exported_priv_warning() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["public-dependency"]
+                cargo-features = ["public-dependency"]
 
-            [package]
-            name = "foo"
-            version = "0.0.1"
+                [package]
+                name = "foo"
+                version = "0.0.1"
 
-            [dependencies]
-            priv_dep = "0.1.0"
-        "#,
+                [dependencies]
+                priv_dep = "0.1.0"
+            "#,
         )
         .file(
             "src/lib.rs",
@@ -35,16 +38,10 @@ fn exported_priv_warning() {
 
     p.cargo("build --message-format=short")
         .masquerade_as_nightly_cargo()
-        .with_stderr(
+        .with_stderr_contains(
             "\
-[UPDATING] `[..]` index
-[DOWNLOADING] crates ...
-[DOWNLOADED] priv_dep v0.1.0 ([..])
-[COMPILING] priv_dep v0.1.0
-[COMPILING] foo v0.0.1 ([CWD])
-src/lib.rs:3:13: warning: type `priv_dep::FromPriv` from private dependency 'priv_dep' in public interface
-[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
-"
+src/lib.rs:3:13: warning: type `[..]FromPriv` from private dependency 'priv_dep' in public interface
+",
         )
         .run()
 }
@@ -52,6 +49,7 @@ src/lib.rs:3:13: warning: type `priv_dep::FromPriv` from private dependency 'pri
 #[cargo_test]
 fn exported_pub_dep() {
     if !is_nightly() {
+        // exported_private_dependencies lint is unstable
         return;
     }
     Package::new("pub_dep", "0.1.0")
@@ -62,15 +60,15 @@ fn exported_pub_dep() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["public-dependency"]
+                cargo-features = ["public-dependency"]
 
-            [package]
-            name = "foo"
-            version = "0.0.1"
+                [package]
+                name = "foo"
+                version = "0.0.1"
 
-            [dependencies]
-            pub_dep = {version = "0.1.0", public = true}
-        "#,
+                [dependencies]
+                pub_dep = {version = "0.1.0", public = true}
+            "#,
         )
         .file(
             "src/lib.rs",
@@ -102,8 +100,8 @@ pub fn requires_nightly_cargo() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["public-dependency"]
-        "#,
+                cargo-features = ["public-dependency"]
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -116,7 +114,8 @@ error: failed to parse manifest at `[..]`
 
 Caused by:
   the cargo feature `public-dependency` requires a nightly version of Cargo, but this is the `stable` channel
-See https://doc.rust-lang.org/book/appendix-07-nightly-rust.html for more information about Rust release channels.
+  See https://doc.rust-lang.org/book/appendix-07-nightly-rust.html for more information about Rust release channels.
+  See https://doc.rust-lang.org/[..]cargo/reference/unstable.html#public-dependency for more information about using this feature.
 "
         )
         .run()
@@ -133,13 +132,13 @@ fn requires_feature() {
             "Cargo.toml",
             r#"
 
-            [package]
-            name = "foo"
-            version = "0.0.1"
+                [package]
+                name = "foo"
+                version = "0.0.1"
 
-            [dependencies]
-            pub_dep = { version = "0.1.0", public = true }
-        "#,
+                [dependencies]
+                pub_dep = { version = "0.1.0", public = true }
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -154,7 +153,7 @@ error: failed to parse manifest at `[..]`
 Caused by:
   feature `public-dependency` is required
 
-consider adding `cargo-features = [\"public-dependency\"]` to the manifest
+  consider adding `cargo-features = [\"public-dependency\"]` to the manifest
 ",
         )
         .run()
@@ -170,15 +169,15 @@ fn pub_dev_dependency() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["public-dependency"]
+                cargo-features = ["public-dependency"]
 
-            [package]
-            name = "foo"
-            version = "0.0.1"
+                [package]
+                name = "foo"
+                version = "0.0.1"
 
-            [dev-dependencies]
-            pub_dep = {version = "0.1.0", public = true}
-        "#,
+                [dev-dependencies]
+                pub_dep = {version = "0.1.0", public = true}
+            "#,
         )
         .file(
             "src/lib.rs",
